// Whats a disembodied cache you ask?  A disembodied cache is a cache
// without any real memory behind it.  This is basically how the old
// Simple-scalar cache used to do it.  It's totally useless
// (well close to useless)
// for MP coherency studies, but it is o.k. for basic architecture studies.
// This Disembodied cache supports LRU replacement.
// 
// To be useful, caches must look like memory systems.


class	DisembodiedCache	:
	public	MemorySystem
	{
	protected:
		Cache			*theCache;
	public:
		MemorySystem	*AttachedMemorySystem;
		Component		*AttachedUpStreamDevice;
		int				FrontSideBusStartupCost;
		int				FrontSideBusTransferTime;
		int				BackSideBusStartupCost;
		int				BackSideBusTransferTime;
		int				LineSize;
		int				BankSize;
		int				Associativity;
		int				FrontSideBusBusy;
		int				BackSideBusBusy;
		int				AccessLatency;
		bool			WriteBack;
		Resource		FrontSideBus;
		Resource		SRAM;
		Counter			FrontSideBusBusyCycles;
		Counter			FrontSideBusIdleCycles;
	public:

		uint8	ReadMemory8(SimAddress);
		uint16	ReadMemory16(SimAddress);
		uint32	ReadMemory32(SimAddress);
		bool	WriteMemory8(SimAddress, uint8);
		bool	WriteMemory16(SimAddress, uint16);
		bool	WriteMemory32(SimAddress, uint32);
	public:
		virtual	bool	Initialize(
						uint32	aLineSize,
						uint32	aBankSize,
						uint32	aAssociativity,
						bool	aWriteBack,
						int		aFrontSideBusStartupTime,
						int		aFrontSideBusTransferTime,
						int		aAccessLatency);
	public:
				DisembodiedCache();
		virtual	~DisembodiedCache();
		virtual	bool	UpdateCache(bool	IsWrite, SimAddress, int	Bytes);
		bool	ConnectToDownStreamComponent(MemorySystem	*);
		bool	ConnectToUpStreamComponent(Component	*);
		bool	NotifyAboutStall(Component	*From, int	Cycles);
		virtual	bool	PrintConfiguration(int Format=HumanFormat,
							FILE *fp=stdout);
		virtual	bool	PrintStatistics(int Format=HumanFormat,
							FILE *fp=stdout);
		int		SimulateOneClockCycle();
		bool	ScheduleOperation(bool	IsWrite, SimAddress,
					int	Bytes, int	StartAfter, int	*UntilStart,
					int *UntilFinish);
	};

