// The Cache coherent mechanisms implement an Exclusive/Read-only
// type of protocol.  Hence, cache lines are either exclusivelly
// owned, or are read-only copies.  Read-accesses will cause
// a exclusive ownership of a cache-line to become read-only in
// all caches (that is, if the cache-line is not exclusivelly owned
// by the same).



class	DisembodiedSnoopyCache;

class	DisembodiedSnoopyControlUnit	:
	public	SnoopyControlUnit
	{
	public:
		bool	MustWriteToMemory;
	public:
		MemorySystem					*AttachedMemorySystem;
		Array<SnoopyCache*>				*ConnectedCaches;
		int		BufferedCacheLine;
		int		BufferedCacheLineValid;
		int		LineSize;
		int		ShiftAmount;
		int		FrontSideBusStartupCost;
		int		FrontSideBusTransferTime;
		int		FrontSideBusBusy;
		int		BackSideBusBusy;
		Resource	FrontSideBus;
		Counter	FrontSideBusBusyCycles;
		Counter	FrontSideBusIdleCycles;
		Counter	ExclusiveAccessRequests;
		Counter	ReadOnlyAccessRequests;
	public:
		uint8	ReadMemory8(SimAddress);
		uint16	ReadMemory16(SimAddress);
		uint32	ReadMemory32(SimAddress);
		bool	WriteMemory8(SimAddress, uint8);
		bool	WriteMemory16(SimAddress, uint16);
		bool	WriteMemory32(SimAddress, uint32);

				DisembodiedSnoopyControlUnit();
		virtual	~DisembodiedSnoopyControlUnit();
		bool	PrintConfiguration(int Format=HumanFormat, FILE *fp=stdout);
		bool	PrintStatistics(int Format=HumanFormat, FILE *fp=stdout);
		bool	ScheduleOperation(bool	IsWrite, SimAddress, int	Bytes,
					int		StartAfter, int	*UntilStart, int	*UntilEnd);
		bool	ScheduleSnoopyWriteOperation(SnoopyCache *From,
					SimAddress, int	Bytes,
					int		StartAfter, int	*UntilStart, int	*UntilEnd);
		bool	ConnectMemorySystem(MemorySystem	*m);
		bool	AddConnectedCache(SnoopyCache	*c);
		bool	GainExclusiveAccess(SnoopyCache *, SimAddress, int);
		bool	GainReadOnlyAccess(SnoopyCache *, SimAddress, int);
		bool	NotifyAboutStall(Component *, int);
		int		SimulateOneClockCycle();
		bool	Initialize(
			int		LineSize,
			int		aFrontSideBusStartupCost,
			int		aFrontSideBusTransferTime);
	};



