#	include	"Simulator.h"

#	define	TranslateVirtualToPhysical(xxx)	\
		( ((xxx) & (~PageMask)) | (PhysicalPages[(xxx)>>PageShift]<<PageShift) )

MemoryManagementUnit	::	MemoryManagementUnit ()
	{
	PageSize = Pages = PageMask = TLBSize = PageShift = 0;
	PhysicalPages = NULL;
	TLBPositions = NULL;
	TLBLRUValues = NULL;
	TLBVirtualPages = NULL;
	}

MemoryManagementUnit	::	~MemoryManagementUnit ()
	{
	if ( PhysicalPages )	delete PhysicalPages;
	if ( TLBPositions )		delete TLBPositions;
	if ( TLBLRUValues )		delete TLBLRUValues;
	if ( TLBVirtualPages )	delete TLBVirtualPages;
	}

bool
MemoryManagementUnit	::	AttachMemorySystem (
	MemorySystem	*ms)
	{
	AttachedMemorySystem = ms;
	return ( true );
	}

bool
MemoryManagementUnit	::	PrintConfiguration (
	int		Format,
	FILE	*fp)
	{
	PrintValue(Format, fp, "TLB-Size", TLBSize);
	PrintValue(Format, fp, "TLB-PageSize", PageSize);
	return(true);
	}

bool
MemoryManagementUnit	::	PrintStatistics (
	int		Format,
	FILE	*fp)
	{
	PrintValue(Format, fp, "TLB-Hits", TLBHits);
	PrintValue(Format, fp, "TLB-Misses", TLBMisses);
	return(true);
	}

bool
MemoryManagementUnit	::	Initialize (
	uint	aPageSize,
	uint	aPages,
	uint	aTLBSize,
	char	*MemoryMap,
	uint	aTranslationDelay,
	uint	aTLBMissDelay)
	{
	TranslationDelay = aTranslationDelay;
	TLBMissDelay = aTLBMissDelay;

	TLBHits = TLBMisses = 0;

	PageSize = aPageSize;
	Pages = aPages;
	// determine page mask & shift;

	int	Index = 1, Shift = 0;
	while ( Index < PageSize )
		{
		Index =	Index << 1;
		++Shift;
		}

	int64	x = (Pages * PageSize - 1);
	PageMask = (uint) x;
	PageMask = PageMask >> Shift;
	PageMask = PageMask << Shift;
	PageShift = Shift;
	Pages = aPages;
	TLBSize = aTLBSize;

	// Load up memory map.
	FILE	*fp = fopen(MemoryMap, "rb");
	if(!fp)
		return ( false );

	PhysicalPages = new uint[Pages];

	fread(PhysicalPages, sizeof(uint), Pages, fp);
	fclose(fp);

	TLBPositions = new int[Pages];
	TLBLRUValues = new int[TLBSize];
	TLBValidBits = new int[TLBSize];
	TLBVirtualPages = new uint[TLBSize];

	Index = 0;
	while ( Index < TLBSize )
		{
		TLBLRUValues[Index] = Index;
		TLBValidBits[Index] = 0;
		TLBVirtualPages[Index] = 0;
		++Index;
		}
	Index = 0;
	while ( Index < Pages )
		{
		TLBPositions[Index] = -1;
		++Index;
		}

	return ( true );
	}

uint8
MemoryManagementUnit	::	ReadMemory8 (
	SimAddress	sa)
	{
	return(AttachedMemorySystem->ReadMemory8(TranslateVirtualToPhysical(sa)));	
	}

uint16
MemoryManagementUnit	::	ReadMemory16 (
	SimAddress	sa)
	{
	return(AttachedMemorySystem->ReadMemory16(TranslateVirtualToPhysical(sa)));	
	}
	
uint32
MemoryManagementUnit	::	ReadMemory32 (
	SimAddress	sa)
	{
	return(AttachedMemorySystem->ReadMemory32(TranslateVirtualToPhysical(sa)));	
	}

bool
MemoryManagementUnit	::	WriteMemory8 (
	SimAddress	sa,
	uint8		d)
	{
	return(AttachedMemorySystem->WriteMemory8(TranslateVirtualToPhysical(sa), d));	
	}

bool
MemoryManagementUnit	::	WriteMemory16 (
	SimAddress	sa,
	uint16		d)
	{
	return(AttachedMemorySystem->WriteMemory16(TranslateVirtualToPhysical(sa), d));	
	}

bool
MemoryManagementUnit	::	WriteMemory32 (
	SimAddress	sa,
	uint32		d)
	{
	return(AttachedMemorySystem->WriteMemory32(TranslateVirtualToPhysical(sa), d));	
	}

bool
MemoryManagementUnit	::	ScheduleOperation(
	bool		op,
	SimAddress	sa,
	int			Bytes,
	int			StartAfter,
	int			*UntilStart,
	int			*UntilFinish)
	{
	// is this in the TLB?
	sa = sa & 0x7fffffff;
	uint	VPN = ((sa&PageMask) >> PageShift);
	int		Index = 0, Index2, Index3;
	if ( TLBPositions[VPN] < 0 )	// TLB Miss ?
		{
		++TLBMisses;
		StartAfter += TLBMissDelay;
		Index = 0;
		while(Index<TLBSize)
			{
			if(!(TLBValidBits[Index]))
				break;
			if(!(TLBLRUValues[Index]))
				Index2 = Index;
			++Index;
			}
		if(Index<TLBSize)
			Index2 = Index;
		}
	else
		{							// TLB Hit
		++TLBHits;
		StartAfter += TranslationDelay;
		Index2 = TLBPositions[VPN];	
		}
	// Now, update the TLB
	Index3 = TLBLRUValues[Index2];
	TLBValidBits[Index2] = 1;
	TLBLRUValues[Index2] = TLBSize - 1;
	TLBPositions[TLBVirtualPages[Index2]] = -1;

	if ( Index3 != (TLBSize - 1) )
		{
		Index = 0;
		while ( Index < TLBSize )
			{
			if(Index == Index2)
				{
				++Index;
				continue;
				}
			if ( TLBLRUValues[Index] > Index3 )
				--TLBLRUValues[Index];
			++Index;
			}
		}
	TLBVirtualPages[Index2] = VPN;

	// Translated, now send the address out the bus

	return(AttachedMemorySystem->ScheduleOperation(
			op, TranslateVirtualToPhysical(sa), Bytes, StartAfter,
			UntilStart, UntilFinish));
	}


