#	include	"Simulator.h"


Resource	::	Resource ()
	{
	ReferenceTime = 0;
	BusyCycles = 0;
	IdleCycles = 0;
	opt_LargestGap = -1;
	opt_EndTime = -1;
	}

void
Resource	::	Sync()
	{
	int	Difference = (int)(theSimulator->CycleCount - ReferenceTime);
	opt_EndTime -= Difference;
	if(ResourceUseArray.GetSize())	
	  	{
	  	int	Length = ResourceUseArray[0].Length;
	  	while(Difference > ResourceUseArray[0].Length)
			{
			Difference -= Length;
			if (ResourceUseArray[0].Filled)
				BusyCycles += Length;
			else
				IdleCycles += Length;
			ResourceUseArray.RemoveAt(0);
			if(!ResourceUseArray.GetSize())
				break;
			Length = ResourceUseArray[0].Length;
			}
	  	}

	if(ResourceUseArray.GetSize())
		{
		ResourceUseArray[0].Length -= Difference;
		if (ResourceUseArray[0].Filled)
			BusyCycles += Difference;
		else
			IdleCycles += Difference;
		}
	else
		{
		IdleCycles += Difference;
		}

	ReferenceTime = theSimulator->CycleCount;
	}

bool
Resource	::	FillHole (
	int		StartAfter,
	int		Length,
	int		EventId,
	int		*StartsAt,
	int		*EndsAt,
	int		*EndTime)
	{
	int		StartedAt, EndingAt, Index;
	int		Time = 0;
	ResourceUse	RU;

	// Slow this routine down a bit
	opt_LargestGap = -1;
	opt_EndTime = -1;
	Index = 0;
	while( Index < (ResourceUseArray.GetSize()) )
		{
		if (!ResourceUseArray[Index].Filled)
		if (ResourceUseArray[Index].Length > opt_LargestGap)
			opt_LargestGap = ResourceUseArray[Index].Length;
		++Index;
		}

	Index = 0;
	int	OldTime;
	while( Index < (ResourceUseArray.GetSize()) )
		{
		OldTime = Time;

		Time += ResourceUseArray[Index].Length;
		if (ResourceUseArray[Index].Filled || 
			(ResourceUseArray[Index].Length < Length) ||
			( (Time-Length) < StartAfter))
			{
			++Index;
			continue;
			}

		// Found a pocket.  Schedule right after Index
		RU.Length = Length;
		RU.Filled = true;
		RU.EventId = EventId;
		if (OldTime < StartAfter)
			{
			int len;
			*StartsAt = StartAfter;
			ResourceUseArray[Index].Length -= Length;
			ResourceUseArray.InsertAt(RU, Index+1);
			if (OldTime + ResourceUseArray[Index].Length > StartAfter)
				{
				RU.Length = OldTime + ResourceUseArray[Index].Length - 
								StartAfter;
				RU.Filled = false;

				if(RU.Length > opt_LargestGap)
					opt_LargestGap = RU.Length;
				ResourceUseArray.InsertAt(RU, Index+2);
				ResourceUseArray[Index].Length = StartAfter - OldTime;
				}
			}
		else
			{
			*StartsAt = OldTime;
			ResourceUseArray[Index].Length -= Length;
			if (!ResourceUseArray[Index].Length)
				ResourceUseArray.RemoveAt(Index);
			ResourceUseArray.InsertAt(RU, Index);
			}

		(*EndsAt)	= *StartsAt + Length;
		return(true);
		}

	(*EndTime) = Time;
	return ( false );
	}

bool
Resource	::	RequestNewSchedule (
	int		StartAfter,
	int		Length,
	int		EventId,
	int		*StartsAt,
	int		*EndsAt)
	{
	// Basic idea is look for a "hole" from start to finish where
	// we might schedule this task.
	int		StartedAt, EndingAt, Index;
	int		Time = 0, OldTime;
	ResourceUse	RU;

	if (Length <= 0)
		{
		(*StartsAt) = (*EndsAt) = StartAfter;
		++BusyCycles;
		IdleCycles = theSimulator->CycleCount - BusyCycles;
		return( true );
		}

	Sync();

	if( opt_EndTime < 0)
		{
		if(FillHole(StartAfter, Length, EventId, StartsAt, EndsAt, &Time))
			return(true);
		}
	else
		{
		if(opt_LargestGap >= Length)
			{
			if(FillHole(StartAfter, Length, EventId, StartsAt, EndsAt, &Time))
				return(true);
			}
		else
			{
			Time = opt_EndTime;
			}
		}	
	
	// No pocket found.  Schedule at the end
	if (Time < StartAfter)
		{
		RU.Length = StartAfter - Time;
		RU.Filled = false;
		if(RU.Length > opt_LargestGap)
			opt_LargestGap = RU.Length;
		ResourceUseArray.Add(RU);
		Time = StartAfter;
		}

	RU.Length = Length;
	RU.Filled = true;
	RU.EventId = EventId;
	ResourceUseArray.Add(RU);

	(*StartsAt)	= Time;
	(*EndsAt)	= Time+Length;

	opt_EndTime = Time+Length;

	return ( true );
	}

int
Resource	::	SimulateOneClockCycle (
	bool	IsDrainTick)
	{
	if(ResourceUseArray.GetSize())
		{
		while(ResourceUseArray.GetSize())
			{
			ResourceUseArray[0].Length -= 1;
			if (ResourceUseArray[0].Length <= 0)
				ResourceUseArray.RemoveAt(0); 
			}
		}
	else
		ReferenceTime = 0;
	return(1);
	}

bool
Resource	::	Busy ()
	{

	int	Difference = (int)(theSimulator->CycleCount - ReferenceTime);

	if(ResourceUseArray.GetSize())	
	while(Difference > ResourceUseArray[0].Length)
		{
		Difference -= ResourceUseArray[0].Length;
		ResourceUseArray.RemoveAt(0);
		if(!ResourceUseArray.GetSize())
			break;
		}

	if(ResourceUseArray.GetSize())
		ResourceUseArray[0].Length -= Difference;
	ReferenceTime = theSimulator->CycleCount;
	

	if(ResourceUseArray.GetSize()==0)
		return(false);
	if(ResourceUseArray[0].Filled == false)
		return(false);
	return(true);
	}

void
Resource	::	Clear()
	{
	ResourceUseArray.Clear();
	}

