#	include	"Simulator.h"

// Additionally, this file has several system depedent requirements.
// For now, I don't particularly care about compatibility except Linux
// and possibly SolarisX86 so drop all the other stuff
extern	"C"
	{
#	include	<fcntl.h>
#	include	<sys/types.h>
#	include	<sys/param.h>
#	include	<errno.h>
#	include	<time.h>
#	include	<sys/time.h>
#	include	<sys/resource.h>
#	include	<signal.h>
#	include	<sys/file.h>
#	include	<sys/stat.h>
#	include	<sys/uio.h>
#	include	<setjmp.h>
#	include	<sys/times.h>
#	include	<limits.h>
#	include	<sys/ioctl.h>
#	include	<termio.h>
#	include	<termios.h>
#	include	<dirent.h>
#	include	<sys/dir.h>

#include "SysCallDefinations.h"

/* open(2) flags translation table for SimpleScalar target */
struct {
  int ss_flag;
  int local_flag;
} ss_flag_table[] = {
  /* target flag */	/* host flag */
  { SS_O_RDONLY,	O_RDONLY },
  { SS_O_WRONLY,	O_WRONLY },
  { SS_O_RDWR,		O_RDWR },
  { SS_O_APPEND,	O_APPEND },
  { SS_O_CREAT,		O_CREAT },
  { SS_O_TRUNC,		O_TRUNC },
  { SS_O_EXCL,		O_EXCL },
  { SS_O_NONBLOCK,	O_NONBLOCK },
  { SS_O_NOCTTY,	O_NOCTTY },
#ifdef O_SYNC
  { SS_O_SYNC,		O_SYNC },
#endif
};
#define SS_NFLAGS	(sizeof(ss_flag_table)/sizeof(ss_flag_table[0]))

	}

bool
SystemTrapHandler	::	HandleSystemTrap(
	Processor		*FromProcessor,
	SimInstruction	*AtInstruction)
	{
	SimRegister	*IntegerRegisters = FromProcessor->GetIntegerRegisterSet();

	switch ( IntegerRegisters[2] )
		{
		case	SS_SYS_exit:
			return(Perform_SS_SYS_exit(FromProcessor, IntegerRegisters));
		case	SS_SYS_fork:
			return(Perform_SS_SYS_fork(FromProcessor, IntegerRegisters));
		case	SS_SYS_vfork:
			return(Perform_SS_SYS_vfork(FromProcessor, IntegerRegisters));
		case	SS_SYS_read:
			return(Perform_SS_SYS_read(FromProcessor, IntegerRegisters));
		case	SS_SYS_write:
			return(Perform_SS_SYS_write(FromProcessor, IntegerRegisters));
		case	SS_SYS_open:
			return(Perform_SS_SYS_open(FromProcessor, IntegerRegisters));
		case	SS_SYS_close:
			return(Perform_SS_SYS_close(FromProcessor, IntegerRegisters));
		case	SS_SYS_creat:
			return(Perform_SS_SYS_creat(FromProcessor, IntegerRegisters));
		case	SS_SYS_unlink:
			return(Perform_SS_SYS_unlink(FromProcessor, IntegerRegisters));
		case	SS_SYS_chdir:
			return(Perform_SS_SYS_chdir(FromProcessor, IntegerRegisters));
		case	SS_SYS_chmod:
			return(Perform_SS_SYS_chmod(FromProcessor, IntegerRegisters));
		case	SS_SYS_chown:
			return(Perform_SS_SYS_chown(FromProcessor, IntegerRegisters));
		case	SS_SYS_brk:
			return(Perform_SS_SYS_brk(FromProcessor, IntegerRegisters));
		case	SS_SYS_lseek:
			return(Perform_SS_SYS_lseek(FromProcessor, IntegerRegisters));
		case	SS_SYS_getuid:
			return(Perform_SS_SYS_getuid(FromProcessor, IntegerRegisters));
		case	SS_SYS_access:
			return(Perform_SS_SYS_access(FromProcessor, IntegerRegisters));
		case	SS_SYS_stat:
			return(Perform_SS_SYS_stat(FromProcessor, IntegerRegisters));
		case	SS_SYS_lstat:
			return(Perform_SS_SYS_lstat(FromProcessor, IntegerRegisters));
		case	SS_SYS_dup:
			return(Perform_SS_SYS_dup(FromProcessor, IntegerRegisters));
		case	SS_SYS_pipe:
			return(Perform_SS_SYS_pipe(FromProcessor, IntegerRegisters));
		case	SS_SYS_getgid:
			return(Perform_SS_SYS_getgid(FromProcessor, IntegerRegisters));
		case	SS_SYS_ioctl:
			return(Perform_SS_SYS_ioctl(FromProcessor, IntegerRegisters));
		case	SS_SYS_fstat:
			return(Perform_SS_SYS_fstat(FromProcessor, IntegerRegisters));
		case	SS_SYS_getpagesize:
			return(Perform_SS_SYS_getpagesize(FromProcessor, IntegerRegisters));
		case	SS_SYS_fsync:
			return(Perform_SS_SYS_fsync(FromProcessor, IntegerRegisters));
		case	SS_SYS_dup2:
			return(Perform_SS_SYS_dup2(FromProcessor, IntegerRegisters));
		case	SS_SYS_fcntl:
			return(Perform_SS_SYS_fcntl(FromProcessor, IntegerRegisters));
		case	SS_SYS_select:
			return(Perform_SS_SYS_select(FromProcessor, IntegerRegisters));
		case	SS_SYS_gettimeofday:
			return(Perform_SS_SYS_gettimeofday(FromProcessor,IntegerRegisters));
		default:
			return(Perform_Unimplemented(FromProcessor, IntegerRegisters));
		}
	return ( false );
	}
	

bool
SystemTrapHandler	::	Perform_SS_SYS_exit(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	// not sure how to handle exit yet.
	FromProcessor->Halt();
	return ( false );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_fork(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[7] = 1;
	regs_R[2] = 1;
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_vfork(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[7] = 1;
	regs_R[2] = 1;
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_read(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	*TemporaryBuffer;

	//printf("syscall: read(%d, %8.8x, %d)\n", regs_R[4], regs_R[5], regs_R[6]);
	fflush(stdout);
	if(regs_R[6] < 0)
		{
		printf("Invalid, asking to read %d bytes\n", regs_R[6]);
		regs_R[2] = 1;
		regs_R[7] = 1;
		return ( false );
		}

	if (!(TemporaryBuffer = new char[regs_R[6]]))
		{
		theSimulator->PrintError("out of memory in SYS_read\n");
		regs_R[2] = 1;
		regs_R[7] = 1;
		return ( false );
		}

	regs_R[2] = read(regs_R[4], TemporaryBuffer, regs_R[6]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}

	FromProcessor->GetDataMemorySystem()->CopyFromHost(
		(void*)regs_R[5], (void*)TemporaryBuffer, (uint)regs_R[2]);

	delete TemporaryBuffer;
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_write(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	*TemporaryBuffer;

	//printf("syscall: write(%d, %8.8x, %d)\n", regs_R[4],regs_R[5], regs_R[6]);
	fflush(stdout);
	if(regs_R[6] < 0)
		{
		printf("Invalid, asking to write %d bytes\n", regs_R[6]);
		regs_R[2] = 1;
		regs_R[7] = 1;
		return ( false );
		}
	if((regs_R[6] < 0)||(regs_R[6]>1000))
		regs_R[6] = 10;

	if (!(TemporaryBuffer = new char[regs_R[6]]))
		{
		theSimulator->PrintError("out of memory in SYS_write\n");
		regs_R[2] = 1;
		regs_R[7] = 1;
		return ( false );
		}


	FromProcessor->GetDataMemorySystem()->CopyToHost(
		(void*)TemporaryBuffer, (void*)regs_R[5], (uint)regs_R[6]);
	regs_R[2] = write(regs_R[4], TemporaryBuffer, regs_R[6]);
	//regs_R[2] = regs_R[6];
	
	if (regs_R[2] == regs_R[6])
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}

	FromProcessor->GetDataMemorySystem()->CopyFromHost(
		(void*)regs_R[5], (void*)TemporaryBuffer, (uint)regs_R[2]);

	delete TemporaryBuffer;
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_open(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];
	uint	Index;
	int		ss_flags = regs_R[5];
	int		local_flags = 0;

	for (Index=0; Index<SS_NFLAGS; Index++)
		{
		if (ss_flags & ss_flag_table[Index].ss_flag)
			{
			ss_flags &= ~ss_flag_table[Index].ss_flag;
			local_flags |= ss_flag_table[Index].local_flag;
			}
		}
	if (ss_flags != 0)
		theSimulator->PrintError(
			"syscall: open: cannot decode flags: 0x%08x", ss_flags);

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = open(Filename, local_flags,  regs_R[6]);
	
	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_close(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	if(regs_R[4]<3)	// lie about closing stdin/stdout
		{
		regs_R[2] = 0;
		regs_R[7] = 0;
		return ( true );
		}
	regs_R[2] = close(regs_R[4]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_creat(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = creat(Filename, regs_R[5]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_unlink(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = unlink(Filename);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_chdir(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Directory[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Directory, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = chdir(Directory);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_chmod(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = chmod(Filename, regs_R[5]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_chown(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = chown(Filename, regs_R[5], regs_R[6]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_brk(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = 0;
	regs_R[7] = 0;
	return ( false );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_lseek(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = lseek(regs_R[4], regs_R[5], regs_R[6]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_getuid(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = getpid();
	regs_R[3] = geteuid();
	
	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_access(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = access(Filename, regs_R[5]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_stat(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];
	struct ss_statbuf ss_sbuf;
	struct stat sbuf;

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = stat(Filename, &sbuf);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	ss_sbuf.ss_st_dev = sbuf.st_dev;
	ss_sbuf.ss_st_ino = sbuf.st_ino;
	ss_sbuf.ss_st_mode = sbuf.st_mode;
	ss_sbuf.ss_st_nlink = sbuf.st_nlink;
	ss_sbuf.ss_st_uid = sbuf.st_uid;
	ss_sbuf.ss_st_gid = sbuf.st_gid;
	ss_sbuf.ss_st_rdev = sbuf.st_rdev;
	ss_sbuf.ss_st_size = sbuf.st_size;
	ss_sbuf.ss_st_atime = sbuf.st_atime;
	ss_sbuf.ss_st_mtime = sbuf.st_mtime;
	ss_sbuf.ss_st_ctime = sbuf.st_ctime;
	ss_sbuf.ss_st_blksize = sbuf.st_blksize;
	ss_sbuf.ss_st_blocks = sbuf.st_blocks;

	FromProcessor->GetDataMemorySystem()->CopyFromHost(
		(void*)regs_R[5], &ss_sbuf, sizeof(struct ss_statbuf));

	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_lstat(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	char	Filename[MAXBUFSIZE];
	struct ss_statbuf ss_sbuf;
	struct stat sbuf;

	FromProcessor->GetDataMemorySystem()->CopyStringToHost(
		Filename, (void*)regs_R[4], MAXBUFSIZE);

	regs_R[2] = lstat(Filename, &sbuf);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	ss_sbuf.ss_st_dev = sbuf.st_dev;
	ss_sbuf.ss_st_ino = sbuf.st_ino;
	ss_sbuf.ss_st_mode = sbuf.st_mode;
	ss_sbuf.ss_st_nlink = sbuf.st_nlink;
	ss_sbuf.ss_st_uid = sbuf.st_uid;
	ss_sbuf.ss_st_gid = sbuf.st_gid;
	ss_sbuf.ss_st_rdev = sbuf.st_rdev;
	ss_sbuf.ss_st_size = sbuf.st_size;
	ss_sbuf.ss_st_atime = sbuf.st_atime;
	ss_sbuf.ss_st_mtime = sbuf.st_mtime;
	ss_sbuf.ss_st_ctime = sbuf.st_ctime;
	ss_sbuf.ss_st_blksize = sbuf.st_blksize;
	ss_sbuf.ss_st_blocks = sbuf.st_blocks;

	FromProcessor->GetDataMemorySystem()->CopyFromHost(
		(void*)regs_R[5], &ss_sbuf, sizeof(struct ss_statbuf));

	return ( true );
	}


bool
SystemTrapHandler	::	Perform_SS_SYS_dup(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = dup(regs_R[4]);
	
	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_pipe(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	int		fd[2];

	FromProcessor->GetDataMemorySystem()->CopyToHost(
		(void*)fd, (void*)regs_R[4], 2*sizeof(int));

	regs_R[7] = pipe(fd);

	regs_R[2] = fd[0];
	regs_R[3] = fd[1];

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_getgid(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = getgid();
	regs_R[3] = getegid();
	
	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_ioctl (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	//regs_R[2] = 1;
	//regs_R[7] = 1;
	return(Perform_Unimplemented(FromProcessor, regs_R));
	//return ( false );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_fstat (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	struct	ss_statbuf	ss_sbuf;
	struct	stat	sbuf;

	regs_R[2] = fstat(regs_R[4], &sbuf);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}

	ss_sbuf.ss_st_dev = sbuf.st_dev;
	ss_sbuf.ss_st_ino = sbuf.st_ino;
	ss_sbuf.ss_st_mode = sbuf.st_mode;
	ss_sbuf.ss_st_nlink = sbuf.st_nlink;
	ss_sbuf.ss_st_uid = sbuf.st_uid;
	ss_sbuf.ss_st_gid = sbuf.st_gid;
	ss_sbuf.ss_st_rdev = sbuf.st_rdev;
	ss_sbuf.ss_st_size = sbuf.st_size;
	ss_sbuf.ss_st_atime = sbuf.st_atime;
	ss_sbuf.ss_st_mtime = sbuf.st_mtime;
	ss_sbuf.ss_st_ctime = sbuf.st_ctime;
	ss_sbuf.ss_st_blksize = sbuf.st_blksize;
	ss_sbuf.ss_st_blocks = sbuf.st_blocks;

	FromProcessor->GetDataMemorySystem()->CopyFromHost(
		(void*)regs_R[5], &ss_sbuf, sizeof(struct ss_statbuf));

	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_getpagesize (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = 4096;
	regs_R[7] = 0;

	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_fsync (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = fsync(regs_R[4]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_dup2 (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = dup2(regs_R[4], regs_R[5]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_fcntl (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	regs_R[2] = fcntl(regs_R[4], regs_R[5], regs_R[6]);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}
	return ( true );
	}

bool
SystemTrapHandler	::	Perform_SS_SYS_select (
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	fd_set	readfd, writefd, exceptfd;
	fd_set	*readfdp, *writefdp, *exceptfdp;
	struct	timeval	timeout, *timeoutp;
	SimRegister	param5;

	readfdp = writefdp = exceptfdp = NULL;

	FromProcessor->GetDataMemorySystem()->CopyToHost(
		(void*)&param5, (void*)(regs_R[29]+16), sizeof(SimRegister));
	
	if(regs_R[5])
		{
		FromProcessor->GetDataMemorySystem()->CopyToHost(
			(void*)&readfd, (void*)regs_R[5], sizeof(SimRegister));
		readfdp = &readfd;
		}
	if(regs_R[6])
		{
		FromProcessor->GetDataMemorySystem()->CopyToHost(
			(void*)&writefd, (void*)regs_R[6], sizeof(SimRegister));
		writefdp = &writefd;
		}
	if(regs_R[7])
		{
		FromProcessor->GetDataMemorySystem()->CopyToHost(
			(void*)&exceptfd, (void*)regs_R[7], sizeof(SimRegister));
		exceptfdp = &exceptfd;
		}

	// ignore timeout value, *always* poll.
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	timeoutp = &timeout;

	regs_R[2] = select(regs_R[4],
		readfdp, writefdp, exceptfdp, timeoutp);

	if (regs_R[2] != -1)
		regs_R[7] = 0;
	else
		{
		regs_R[2] = errno;
		regs_R[7] = 1;
		}

	if(regs_R[5])
		FromProcessor->GetDataMemorySystem()->CopyFromHost(
			(void*)regs_R[5], (void*)&readfd, sizeof(SimRegister));
	if(regs_R[6])
		FromProcessor->GetDataMemorySystem()->CopyFromHost(
			(void*)regs_R[6], (void*)&writefd, sizeof(SimRegister));
	if(regs_R[7])
		FromProcessor->GetDataMemorySystem()->CopyFromHost(
			(void*)regs_R[7], (void*)&exceptfd, sizeof(SimRegister));

	return ( true );
	}


bool
SystemTrapHandler	::	Perform_SS_SYS_gettimeofday(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	// regs_R[4] contains a pointer with a timeval struct
	regs_R[2] = 1;
	regs_R[7] = 1;
	return(true);
	}

bool
SystemTrapHandler	::	Perform_Unimplemented(
	Processor	*FromProcessor,
	SimRegister	*regs_R)
	{
	printf("Unimplemented sys-call %d\n", regs_R[2]);
	regs_R[2] = 1;
	regs_R[7] = 1;
	return(true);
	}


