#	include	"Simulator.h"

// Various configuraitons parameters

uint	NumberOfProcessors = 1, StackSize = 0x800000;
uint	L1IBanks = 4, L1ILineSize = 128, L1IBankSize = 32;
bool	L1IWriteBack = false;
uint	L1IFSBSC = 0, L1IFSBBS = 1;
uint	L1DBanks = 2, L1DLineSize = 128, L1DBankSize = 64;
bool	L1DWriteBack = false;
uint	L1DFSBSC = 0, L1DFSBBS = 1;
uint	L2CBanks = 4, L2CLineSize = 128, L2CBankSize = 2048;
bool	L2CWriteBack = true;
uint	L2CFSBSC = 0, L2CFSBBS = 1;
uint	SNOOPYFSBSC = 4, SNOOPYFSBBS = 2;
uint	DRAMSC = 60, DRAMBS = 10, DRAMPageSize = 4096;
int		OutputFormat = HumanFormat;
char	OutputFile[1024] = "stdout";
uint	L1IAccess = 0, L1DAccess = 0, L2Access = 12;
uint	MMUPageSize	= 0x1000, TLBSize = 64, TLBMiss = 70, TLBHit = 1;
char	MMUFile[1024] = "directmap.mmu";

CommandLineArgument	Args[]	=
	{
		{	"Processors",		clauInteger,	&NumberOfProcessors },
		{	"StackSize",		clauInteger,	&StackSize },
		{	"L1-I-Banks",		clauInteger,	&L1IBanks },
		{	"L1-I-LineSize",	clauInteger,	&L1ILineSize },
		{	"L1-I-BankSize",	clauInteger,	&L1IBankSize },
		{	"L1-I-WriteBack",	claBool,		&L1IWriteBack },
		{	"L1-I-FSB-SC",		clauInteger,	&L1IFSBSC },
		{	"L1-I-FSB-BS",		clauInteger,	&L1IFSBBS },
		{	"L1-I-Access",		clauInteger,	&L1IAccess },
		{	"L1-D-Banks",		clauInteger,	&L1DBanks },
		{	"L1-D-LineSize",	clauInteger,	&L1DLineSize },
		{	"L1-D-BankSize",	clauInteger,	&L1DBankSize },
		{	"L1-D-WriteBack",	claBool,		&L1DWriteBack },
		{	"L1-D-FSB-SC",		clauInteger,	&L1DFSBSC },
		{	"L1-D-FSB-BS",		clauInteger,	&L1DFSBBS },
		{	"L1-D-Access",		clauInteger,	&L1DAccess },
		{	"L2-C-Banks",		clauInteger,	&L2CBanks },
		{	"L2-C-LineSize",	clauInteger,	&L2CLineSize },
		{	"L2-C-BankSize",	clauInteger,	&L2CBankSize },
		{	"L2-C-WriteBack",	claBool,		&L2CWriteBack },
		{	"L2-C-FSB-SC",		clauInteger,	&L2CFSBSC },
		{	"L2-C-FSB-BS",		clauInteger,	&L2CFSBBS },
		{	"L2-Access",		clauInteger,	&L2Access },
		{	"SNOOPY-FSB-SC",	clauInteger,	&SNOOPYFSBSC },
		{	"SNOOPY-FSB-BS",	clauInteger,	&SNOOPYFSBSC },
		{	"DRAM-SC",			clauInteger,	&DRAMSC },
		{	"DRAM-BS",			clauInteger,	&DRAMBS },
		{	"DRAM-PageSize",	clauInteger,	&DRAMPageSize },
		{	"MMU-PageSize",		clauInteger,	&MMUPageSize },
		{	"MMU-TLB-Size",		clauInteger,	&TLBSize },
		{	"MMU-TLB-MissCost",	clauInteger,	&TLBMiss },	
		{	"MMU-TLB-HitCost",	clauInteger,	&TLBHit },
		{	"MMU-Map",			claString,		MMUFile },
		{	"OutputFormat",		claInteger,		&OutputFormat },
		{	"OutputFile",		claString,		OutputFile },
		{	NULL,				clauInteger, NULL }
	};

Simulator	*theSimulator;	// must be a global and must be named theSimulator

int
main (
	int		argc,
	char	*argv[],
	char	*argp[])
	{
	// usage: sim3 program [args..]
	int	Index, Processors, vargc;
	SimRegister	ProcessorStackSize;
	char	s[1024];

	if(argc<2)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}
	else
	if((vargc = ParseCommandLine(Args, argc, argv))<0)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}
	if((vargc)>=argc)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}	
	// Global Intialization
	InitializeSimMaskToOpCodeTable();

	Processors = NumberOfProcessors;
	ProcessorStackSize = StackSize;

	// Initialize system-board devices
	theSimulator = new Simulator;
	StandardDRAMDevice			*DRAM = new StandardDRAMDevice;
	MPSystemTrapHandler			*STH = new MPSystemTrapHandler;
	DisembodiedSnoopyControlUnit	*SCU = new DisembodiedSnoopyControlUnit;
	DisembodiedCache			*L1Is, *L1Ds;
	DisembodiedSnoopyCache		 *L2s;
	SingleIssueProcessor		*Proc;
	MemoryManagementUnit		*MMU = new MemoryManagementUnit;

	L1Is = new DisembodiedCache[Processors];
	L1Ds = new DisembodiedCache[Processors];
	L2s = new DisembodiedSnoopyCache[Processors];
	Proc = new SingleIssueProcessor[Processors];

	for(Index=0;Index<Processors;Index++)
		{
		sprintf(s, "Processor-%d-L1-I", Index);
		L1Is[Index].SetName(s);
		L1Is[Index].Initialize(
							L1ILineSize, L1IBankSize, L1IBanks,
							L1IWriteBack,
							L1IFSBSC, L1IFSBBS, L1IAccess);

		sprintf(s, "Processor-%d-L1-D", Index);
		L1Ds[Index].SetName(s);
		L1Ds[Index].Initialize(
							L1DLineSize, L1DBankSize, L1DBanks,
							L1DWriteBack,
							L1DFSBSC, L1DFSBBS, L1DAccess);

		sprintf(s, "Processor-%d-L2", Index);
		L2s[Index].SetName(s);
		L2s[Index].Initialize(
							L2CLineSize, L2CBankSize, L2CBanks,
							L2CWriteBack,
							L2CFSBSC, L2CFSBBS, L2Access);

		sprintf(s, "Processor-%d", Index);
		Proc[Index].SetName(s);

		L1Is[Index].ConnectToUpStreamComponent(&Proc[Index]);
		L1Is[Index].ConnectToDownStreamComponent(&L2s[Index]);
		Proc[Index].ConnectInstructionMemory(&L1Is[Index]);

		L1Ds[Index].ConnectToUpStreamComponent(&Proc[Index]);
		L1Ds[Index].ConnectToDownStreamComponent(&L2s[Index]);
		Proc[Index].ConnectDataMemory(&L1Ds[Index]);

		L2s[Index].ConnectToUpStreamComponent(&Proc[Index]);
		L2s[Index].ConnectToDownStreamComponent(SCU);
	
		Proc[Index].ConnectSystemTrapHandler(STH);
		SCU->AddConnectedCache(&L2s[Index]);
		}

	SCU->Initialize( L2CLineSize, SNOOPYFSBSC, SNOOPYFSBBS);
	SCU->SetName("SnoopyControlUnit");

	DRAM->Initialize(0x80000000/DRAMPageSize, DRAMPageSize, DRAMSC, DRAMBS);
 	DRAM->SetName("DRAM");

	MMU->Initialize(MMUPageSize, 0x80000000/MMUPageSize, TLBSize, MMUFile,
			TLBHit, TLBMiss);
	MMU->SetName("MemoryManagementUnit");

	MMU->AttachMemorySystem(DRAM);
	SCU->ConnectMemorySystem(MMU);
	DRAM->Connect(MMU);

	theSimulator->SetName("Machine");

	// The final thing to set is the log-file output
	FILE	*fp = stdout;
	if(strcmp(OutputFile, "stdout"))
		{
		if(!strcmp(OutputFile, "stderr"))
			fp = stderr;
		else
			{
			fp = fopen(OutputFile, "a+");
			if(!fp)
				{
				printf("error creating logfile: %s\n", OutputFile);
				return(1);
				}
			fseek(fp, 0, SEEK_END);
			}
		}	
	// Print the system board devices configuration	
	theSimulator->PrintConfiguration(OutputFormat, fp);
	STH->PrintConfiguration(OutputFormat, fp);
	SCU->PrintConfiguration(OutputFormat, fp);
	for(Index=0;Index<Processors;Index++)
		{
		Proc[Index].PrintConfiguration(OutputFormat, fp);
		L1Is[Index].PrintConfiguration(OutputFormat, fp);
		L1Ds[Index].PrintConfiguration(OutputFormat, fp);
		L2s[Index].PrintConfiguration(OutputFormat, fp);
		}
	DRAM->PrintConfiguration(OutputFormat, fp);
	MMU->PrintConfiguration(OutputFormat, fp);
	
	if(!Loader::LoadFromDevice(
		argv[vargc], DRAM, &Proc[0], &argv[vargc], argp))
		{
		printf("failed to load %s into memory\n", argv[vargc]);
		return(2);
		}
	// Copy some registers from Proc[0] to other proces
	for(Index=1;Index<Processors;Index++)
		{
		*(Proc[Index].GetProgramCounter()) = *(Proc[0].GetProgramCounter());
		Proc[Index].GetIntegerRegisterSet()[29] =
			Proc[Index-1].GetIntegerRegisterSet()[29] - ProcessorStackSize;
		}
	theSimulator->RegisterComponent(theSimulator);
	theSimulator->RegisterComponent(MMU);
	theSimulator->RegisterComponent(DRAM);
	theSimulator->RegisterComponent(SCU);
	theSimulator->RegisterComponent(STH);
	for(Index=0;Index<Processors;Index++)
		{
		theSimulator->RegisterComponent(&Proc[Index]);
		theSimulator->RegisterComponent(&L1Is[Index]);
		theSimulator->RegisterComponent(&L1Ds[Index]);
		theSimulator->RegisterComponent(&L2s[Index]);
		}

	theSimulator->Simulate();

	printf("\n");
	theSimulator->PrintStatistics(OutputFormat, fp);
	STH->PrintStatistics(OutputFormat, fp);
	SCU->PrintStatistics(OutputFormat, fp);

	for(Index=0;Index<Processors;Index++)
		{
		Proc[Index].PrintStatistics(OutputFormat, fp);
		L1Is[Index].PrintStatistics(OutputFormat, fp);
		L1Ds[Index].PrintStatistics(OutputFormat, fp);
		L2s[Index].PrintStatistics(OutputFormat, fp);
		}
	DRAM->PrintStatistics(OutputFormat, fp);
	MMU->PrintStatistics(OutputFormat, fp);

	fprintf(fp, "\n");

	delete[] Proc;
	delete DRAM;
	delete STH;
	delete[] L1Is;
	delete[] L1Ds;
	delete[] L2s;
	delete SCU;
	return(0);	
	}


